### Code accompanying the paper "Cryptographically secure multiparty evaluation
### of system reliability"

################################################################################
### This code compares the results of the global run on Amazon EC2 with the  ###
### ground truth by computing everything unencrypted at full precision       ###
### locally.                                                                 ###
################################################################################

library("ReliabilityTheory")
library("ggplot2")

# Define grid of time points at which to evaluate survival curve
t <- seq(0, 5, length.out=100)

# Define the system and compute the survival signature.
ab <- graph.formula(s -- M -- C1:C2:C3:C4, P1:P2:P3:P4 -- t,
                    C1 -- P1, C2 -- P2, C3 -- P3, C4 -- P4, s -- H -- P3:P4)
ab <- setCompTypes(ab, list("C"=c("C1", "C2", "C3", "C4"),
                            "H"=c("H"),
                            "M"=c("M"),
                            "P"=c("P1", "P2", "P3", "P4")))

survsig <- computeSystemSurvivalSignature(ab)

# Here simulate the data locally simulated in the global test.
set.seed(which(LETTERS=="C"))
C.data <- rexp(15, rate=0.3)
set.seed(which(LETTERS=="H"))
H.data <- rlnorm(10, 1.5, 0.3)
set.seed(which(LETTERS=="M"))
M.data <- rexp(5, rate=0.25)
set.seed(which(LETTERS=="P"))
P.data <- rgamma(20, scale=0.9, shape=3.2)

# Compute the true survival curve using the non-parametric method of Aslett
# (2015)
true.surv <- nonParBayesSystemInference(t, survsig, list("C"=C.data,
                                                         "H"=H.data,
                                                         "M"=M.data,
                                                         "P"=P.data))

# Load the estimate from the global Amazon run using the cryptographically
# secure protocol
load("AmazonEC2-xi_m.RData")

# Plot the survival curve
#cairo_pdf("survivalcurve.pdf", 4.854, 3)
ggplot(data.frame(Time=t,
                  Method=c(rep("Standard", 100), rep("Encrypted", 100)),
                  Probability=c(true.surv, xi.m))) +
  geom_line(aes(x=Time, y=Probability, colour=Method, linetype=Method)) #+ theme_grey(12, "CMU Serif")
#dev.off()
